/*
   Copyright (C) 2002 Dylan Yudaken(d_yud@hotmail.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/

#include "common.h"
GLYPHMETRICSFLOAT gmf[256];	// Storage For Information About Our Font
GLuint base;
CIndivTexture FontText;
//Got most of this from NeHe
GLvoid BuildFont(GLvoid)					// Build Our Bitmap Font
{
	float	cx;											// Holds Our X Character Coord
	float	cy;											// Holds Our Y Character Coord
	int loop;
	base=glGenLists(256);								// Creating 256 Display Lists
	glBindTexture(GL_TEXTURE_2D, FontText.TexID);			// Select Our Font Texture
	for (loop=0; loop<256; loop++)						// Loop Through All 256 Lists
	{
		cx=float(loop%16)/16.0f;						// X Position Of Current Character
		cy=float(loop/16)/16.0f;						// Y Position Of Current Character

		glNewList(base+loop,GL_COMPILE);				// Start Building A List
			glBegin(GL_QUADS);							// Use A Quad For Each Character
				glTexCoord2f(cx,1-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2i(0,0);						// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,1-cy-0.0625f);	// Texture Coord (Bottom Right)
				glVertex2i(16,0);						// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,1-cy);			// Texture Coord (Top Right)
				glVertex2i(16,16);						// Vertex Coord (Top Right)
				glTexCoord2f(cx,1-cy);					// Texture Coord (Top Left)
				glVertex2i(0,16);						// Vertex Coord (Top Left)
			glEnd();									// Done Building Our Quad (Character)
			glTranslated(13,0,0);						// Move To The Right Of The Character
		glEndList();									// Done Building The Display List
	}		
	Plog(&Log,"Font Created\n");
}

GLvoid KillFont(GLvoid)						// Delete The Font
{
 	glDeleteLists(base, 256);				// Delete All 256 Characters
}

GLvoid glPrint(GLint x,GLint y,int set,const char *fmt, ...)				// Custom GL "Print" Routine
{
	char string[128];
	va_list		ap;					// Pointer To List Of Arguments
	GLboolean DTEST		= 	glIsEnabled(GL_DEPTH_TEST);
	GLboolean CFACE		= 	glIsEnabled(GL_CULL_FACE);
	GLboolean Lighting	= 	glIsEnabled(GL_LIGHTING);
	GLboolean Blend		= 	glIsEnabled(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, FontText.TexID);			// Select Our Font Texture
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,Width,0,Height,-1,1);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	
	
	va_start(ap, fmt);										// Parses The String For Variables
	    vsprintf(string, fmt, ap);							// And Converts Symbols To Actual Numbers
	va_end(ap);												// Results Are Stored In Text

	if (set>1)												// Did User Choose An Invalid Character Set?
	{
		set=1;												// If So, Select Set 1 (Italic)
	}
	glTranslated(x,y,0);								// Position The Text (0,0 - Bottom Left)
	glListBase(base-32+(128*set));						// Choose The Font Set (0 or 1)
	glCallLists(strlen(string),GL_BYTE,string);			// Write The Text To The Screen
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing

	if(DTEST)		glEnable(GL_DEPTH_TEST);
	if(CFACE)		glEnable(GL_CULL_FACE);
	if(Lighting)	glEnable(GL_LIGHTING);
	if(!Blend)		glDisable(GL_BLEND);

}
GLvoid glSPrint(float scale,int w,int h, GLint x,GLint y,int set,const char *fmt, ...)				// Smaller GL "Print" Routine
{
	char string[128];
	va_list		ap;					// Pointer To List Of Arguments
	GLboolean DTEST		= 	glIsEnabled(GL_DEPTH_TEST);
	GLboolean CFACE		= 	glIsEnabled(GL_CULL_FACE);
	GLboolean Lighting	= 	glIsEnabled(GL_LIGHTING);
	GLboolean Blend		= 	glIsEnabled(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, FontText.TexID);			// Select Our Font Texture
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,w,0,h,-1,1);								// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	
	
	va_start(ap, fmt);										// Parses The String For Variables
	    vsprintf(string, fmt, ap);							// And Converts Symbols To Actual Numbers
	va_end(ap);												// Results Are Stored In Text

	if (set>1)												// Did User Choose An Invalid Character Set?
	{
		set=1;												// If So, Select Set 1 (Italic)
	}
	glScalef(scale,scale,1);
	glTranslated(x,y,0);								// Position The Text (0,0 - Bottom Left)
	glListBase(base-32+(128*set));						// Choose The Font Set (0 or 1)
	glCallLists(strlen(string),GL_BYTE,string);			// Write The Text To The Screen
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing

	if(DTEST)		glEnable(GL_DEPTH_TEST);
	if(CFACE)		glEnable(GL_CULL_FACE);
	if(Lighting)	glEnable(GL_LIGHTING);
	if(!Blend)		glDisable(GL_BLEND);

}
