/*
   Copyright (C) 2002 Dylan Yudaken(d_yud@hotmail.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/

#include "common.h"

GLuint List;
GLfloat xspace,yspace,zspace;
int hp,wp,dp;
int BoardPos(int x,int y,int z);
void DrawPicCube(GLfloat x,GLfloat y,GLfloat z,GLfloat w,GLfloat h,GLfloat d);
int Score;
int HighScores[10];
char HighScoreName[10][20];

int CPiece::DeleteMem()
{
	if(!memset(&block,0,sizeof(block)))
		return 0;
	return 1;
}
void CPiece::AssignBlock(int *Directions,int size)
{
	DeleteMem();
	int cx,cy,cz;
	cx = cy = cz = rx = ry = rz = 0;
	block[0][0][0] = true;
	for(int c = 0;c<size;c++){
		switch(Directions[c]){
			case DUP:
				if(cy < MAXBLOCKS-1)
				{
					cy++;
				}
				block[cx][cy][cz] = true;
				break;
			case DDOWN:
				if(cy > 0)
				{
					cy--;
				}
				block[cx][cy][cz] = true;
				break;
			case DLEFT:
				if(cx > 0)
				{
					cx--;
				}
				block[cx][cy][cz] = true;
				break;
			case DRIGHT:
				if(cx < MAXBLOCKS-1)
				{
					cx++;
				}
				block[cx][cy][cz] = true;
				break;
			case DFORWARD:
				if(cz > 0)
				{
					cz--;
				}
				block[cx][cy][cz] = true;
				break;
			case DBACKWARD:
				if(cz < MAXBLOCKS-1)
				{
					cz++;
				}
				block[cx][cy][cz] = true;
				break;
		}
	}
	Psize = size;
	if(PDir != NULL)delete[] PDir;
	this->PDir = new int [Psize];
	memcpy(&PDir,Directions,sizeof(Directions));
	px = py = pz = LowX = LowY = LowZ = MaxWidth = MaxHeight = MaxDepth = 0; //Assign 0 to all
	FindMaxDimentions();
	Move(0,0,0);

}
void CPiece::Rotate(RotType r)
{
	int buff[MAXBLOCKS][MAXBLOCKS][MAXBLOCKS];
	unsigned int a,b,c;
	int mx,my,mz;
	mx = my = mz = 0;
	GLBPos retval;
	GLBPos val;
	GLBPos *Nblock = NULL;
	unsigned int PosCount = 0;
	Nblock = new GLBPos [MAXBLOCKS*MAXBLOCKS*MAXBLOCKS];
	memset(&buff,0,sizeof(buff));
	for(a = 0;a<MAXBLOCKS;a++){
		for(b = 0;b<MAXBLOCKS;b++){
			for(c =0;c<MAXBLOCKS;c++){
				if(block[a][b][c]){
					val.x = a;
					val.y = b;
					val.z = c;
					retval = RotatePos(r,val);
					Nblock[PosCount] = retval;
					PosCount++;

				}
			}	
		}
	}
	for(a = 0;a<PosCount;a++){//Find farthest negatives
		if(Nblock[a].x  < mx)
			mx = Nblock[a].x;

		if(Nblock[a].y  < my)
			my = Nblock[a].y;

		if(Nblock[a].z  < mz)
			mz = Nblock[a].z;

	}
	mx = -mx;
	my = -my;
	mz = -mz;
	if(mx  > MAXBLOCKS || my  > MAXBLOCKS || mz  > MAXBLOCKS){
		done = true;
	}
	else{
		for(a = 0;a<PosCount;a++){
			buff[Nblock[a].x+mx][Nblock[a].y+my][Nblock[a].z+mz] = true;
		}
	}
	DeleteMem();
	memcpy(&block,&buff,sizeof(block));
	FindMaxDimentions();
	Move(0,0,0);
	if(Nblock != NULL)delete[] Nblock;
}
GLBPos RotatePos(RotType r,GLBPos p)
{
	#define C270 0
	#define S270 -1
	#define C180 -1
	#define S180 0
	#define  C90 0
	#define  S90 1
	int bx,by,bz ;
	GLBPos buff;
	int x = p.x;
	int y = p.y;
	int z = p.z;
	#define srot S90
	#define crot C90
	switch (r){
	case RotX:
		bx = x;
		by = ((crot*y)+(-srot*z));
		bz = ((srot*y)+(crot*z));
		break;

	case RotMX:
		bx = x;
		by = ((-crot*y)+(srot*z));
		bz = ((-srot*y)+(-crot*z));
		break;

	case RotY:
		bx = (crot*x)+(srot*z);
		by = y;
		bz = (-srot*x)+(crot*z);
		break;

	case RotMY:
		bx = (-crot*x)+(-srot*z);
		by = y;
		bz = (srot*x)+(-crot*z);
		break;

	case RotZ:
		bx = (crot*x)+(-srot*y);
		by = (srot*x)+(crot*y);
		bz = z;
		break;

	case RotMZ:
		bx = (-crot*x)+(srot*y);
		by = (-srot*x)+(-crot*y);
		bz = z;

		break;

	default:
		break;
	}
	
	buff.x = bx;
	buff.y = by;
	buff.z = bz;
	return buff;
}

void CPiece::Draw()
{
#define XT px*zspace
#define YT py*yspace
#define ZT (dp-pz)*xspace
	int x,y,z;
	glEnable(GL_LINE_STIPPLE);
	glLineWidth(1.8f);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LINE_SMOOTH);
	glTranslatef(0,0,DSIZE);
	glTranslatef(XT,YT,ZT);
	glLineStipple(1,43690);
	for(x = 0;x<MAXBLOCKS;x++){
		for(y = 0;y<MAXBLOCKS;y++){
			for(z = 0;z<MAXBLOCKS;z++){
				if(block[x][y][z])
					DrawPicCube((GLfloat)x*zspace,(GLfloat)y*yspace,(GLfloat)z*xspace,zspace,yspace,xspace);
			}
		}
	}
	glEnable(GL_TEXTURE_2D);
	glTranslatef(-(XT),-(YT),-(ZT));
	glDisable(GL_LINE_STIPPLE);
	glLineWidth(1.0f);
	

}
void CPiece::Move(int x,int y,int z)
{
	px += x;
	py += y;
	pz += z;
	
	if(px < 0)
		px = 0;
	if(py < 0)
		py = 0;
	if(MaxDepth > pz+1 )
		pz = MaxDepth;
		
	if(px + MaxWidth > wp)
		px = wp - MaxWidth;
	if(py + MaxHeight > hp)
		py = hp - MaxHeight;
	if(pz > dp){
		pz = dp - MaxDepth;
	}


}
int CPiece::FindMaxDimentions()
{
	int MaxX,MaxY,MaxZ,LX,LY,LZ;
	MaxX = MaxY = MaxZ = 0;
	LX = LY = LZ = MAXBLOCKS;
	for(int x = 0;x<MAXBLOCKS;x++){
		for(int y = 0;y<MAXBLOCKS;y++){
			for(int z = 0;z<MAXBLOCKS;z++){
				if(block[x][y][z]){
					if(x < LX)
						LX = x;
					if(y < LY)
						LY = y;
					if(z < LZ)
						LZ = z;
					if(x > MaxX)
						MaxX = x;
					if(y > MaxY)
						MaxY = y;
					if(z > MaxZ)
						MaxZ = z;
				}
			}
		}
	}
	MaxWidth = MaxX+1;
	MaxHeight = MaxY+1;
	MaxDepth = MaxZ+1;
	LowX = LX;
	LowY = LY;
	LowZ = LZ;
	return 1;
}
void CPiece::Reset()
{
	AssignBlock(this->PDir,this->Psize);
}
void DrawPicCube(GLfloat x,GLfloat y,GLfloat z,GLfloat w,GLfloat h,GLfloat d)
{
	glBegin(GL_QUADS);
		//Back
		glNormal3f( 0.0f, 0.0f, -1.0f);
		glVertex3f(x  ,y+h,z  );	//BTL
		glVertex3f(x+w,y+h,z  );	//BTR
		glVertex3f(x+w,y  ,z  );	//BBR
		glVertex3f(x  ,y  ,z  );	//BBL

		//Top
		glNormal3f( 0.0f, 1.0f, 0.0f);
		glVertex3f(x  ,y+h  ,z);	//TFL
		glVertex3f(x+w,y+h  ,z);	//TFR
		glVertex3f(x+w,y+h  ,z-d);	//TBR
		glVertex3f(x  ,y+h  ,z-d);	//TBL

		//Front

		glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Forward
		glVertex3f(x  ,y+h  ,z-d);	//FTL
		glVertex3f(x+w,y+h  ,z-d);	//FTR
		glVertex3f(x+w,y    ,z-d);	//FBR
		glVertex3f(x  ,y    ,z-d);	//FBL

		//Bottom
		glNormal3f( 0.0f, -1.0f, 0.0f);
		glVertex3f(x  ,y  ,z  );	//BFL
		glVertex3f(x+w,y  ,z  );	//BFR
		glVertex3f(x+w,y  ,z-d);	//BBR
		glVertex3f(x  ,y  ,z-d);	//BBL

		//Left
		glNormal3f( -1.0f, 0.0f, 0.0f);
		glVertex3f(x  ,y+h  ,z  );	//LFT
		glVertex3f(x  ,y+h  ,z-d);	//LBT
		glVertex3f(x  ,y    ,z-d);	//LBB
		glVertex3f(x  ,y    ,z  );	//LFB

		//Right
		glNormal3f( 1.0f, 0.0f, 0.0f);
		glVertex3f(x+w,y+h  ,z  );	//RFT
		glVertex3f(x+w,y+h  ,z-d);	//RBT
		glVertex3f(x+w,y    ,z-d);	//RBB
		glVertex3f(x+w,y    ,z  );	//RFB

	glEnd();
}
void CGame::DrawHelpPiece(CPiece *P)
{
	if(GhostBlock != 1)
		return;
	int answer,p;
	int x,y,z;
#define HXT P->px*zspace
#define HYT P->py*yspace
#define HZT (dp-P->pz)*xspace
	for(z = 0;z<dp;z++)
	{

		//Game->TestPiece(this);
		for(x = 0;x<MAXBLOCKS;x++){
			for(y = 0;y<MAXBLOCKS;y++){
				if(P->block[x][y][P->LowZ]){
					p = BoardPos(x+P->px,y+P->py,P->LowZ+z+1);
					if(P->LowZ+z+1 >= dp)
					{
						answer = z;
						x = MAXBLOCKS;
						y = MAXBLOCKS;
						z = dp;
						break;
					}
					if((board[p])){
						{
							answer = z;
							x = MAXBLOCKS;
							y = MAXBLOCKS;
							z = dp;
							break;
						}
					}
				}
			}
		}
	}
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_STIPPLE);
	glLineWidth(1.8f);
	glLineStipple(1,43690);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LINE_SMOOTH);
	glTranslatef(0,0,DSIZE);
	glTranslatef(HXT,HYT,0);	
	//Pass ONE
	//Render Lines Around The block
	glColor4f(1,1,1,1);
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	//Pass TWO 
	//Render a blended block
	glColor4f(0.5f,0.5f,1,0.5f);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	for(x = 0;x<MAXBLOCKS;x++){
		for(y = 0;y<MAXBLOCKS;y++){
			for(z = 0;z<MAXBLOCKS;z++){
				if(P->block[x][y][z])
					DrawPicCube((GLfloat)x*zspace,(GLfloat)y*yspace,(GLfloat)(z+(dp-dp-answer))*xspace,zspace,yspace,xspace);
			}
		}
	}
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glTranslatef(-(HXT),-(HYT),0);
	glDisable(GL_LINE_STIPPLE);
	glLineWidth(1.0f);

}
int CGame::GenerateLines(int h,int w,int d)
{
	int c;
	static int done;
	this->h = hp = h;
	this->w = wp = w;
	this->d = dp = d;
	xspace = DSIZE/(float)d;
	yspace = USIZE/(float)h;
	zspace = USIZE/(float)w;
	if(!done){
	List = glGenLists(1);
	}
	else{
		glDeleteLists(List,1);
	}

	glNewList(List,GL_COMPILE);
	
		glTranslatef(-HALFUSIZE,-HALFUSIZE,0);
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_LINES);
		//Left.Right,Up,Down sides
			for(c=0;c<h+1;c++){//Horizontal first Left
				glVertex3f(0,(float)c*yspace,0);	
				glVertex3f(0,(float)c*yspace,DSIZE);	
			}
			for(c = 0;c<d+1;c++){//then vetical
				glVertex3f(0,0,(float)c*xspace);
				glVertex3f(0,USIZE,(float)c*xspace);
			}
			for(c=0;c<h+1;c++){//Horizontal first Right
				glVertex3f(USIZE,(float)c*yspace,0);	
				glVertex3f(USIZE,(float)c*yspace,DSIZE);	
			}
			for(c = 0;c<d+1;c++){//then vetical
				glVertex3f(USIZE,0,(float)c*xspace);
				glVertex3f(USIZE,USIZE,(float)c*xspace);
			}
			for(c=0;c<w+1;c++){//Horizontal first Up
				glVertex3f((float)c*zspace,USIZE,0);	
				glVertex3f((float)c*zspace,USIZE,DSIZE);	
			}
			for(c = 0;c<d+1;c++){//then vetical
				glVertex3f(0,    USIZE,(float)c*xspace);
				glVertex3f(USIZE,USIZE,(float)c*xspace);
			}
			for(c=0;c<w+1;c++){//Horizontal first Down
				glVertex3f((float)c*zspace,0,0);	
				glVertex3f((float)c*zspace,0,DSIZE);	
			}
			for(c = 0;c<d+1;c++){//then vetical
				glVertex3f(0,    0,(float)c*xspace);
				glVertex3f(USIZE,0,(float)c*xspace);
			}
			
		glEnd();
		glTranslatef(0,0,-DSIZE);
		glBegin(GL_LINES);		
			for(c=0;c<h+1;c++){//Horizontal first Back
				glVertex3f(0,    (float)c*yspace,DSIZE);	
				glVertex3f(USIZE,(float)c*yspace,DSIZE);	
			}
			for(c = 0;c<w+1;c++){//then vetical
				glVertex3f((float)c*zspace,0,DSIZE);
				glVertex3f((float)c*zspace,USIZE,DSIZE);
			}
			
		glEnd();
		//Lastly the back part
		glEnable(GL_TEXTURE_2D);
	glEndList();
	return 1;
}
void CGame::RenderLines()
{
	glCallList(List);
}

void CGame::CheckGameKeys() //Check Rotation and movement
{
	int PZ = Piece->pz;
	if(keys[SDLK_UP]&& !keyc[SDLK_UP]){
		keyc[SDLK_UP] = true;
		Piece->Move(0,1,0);
		if(TestPiece(Piece))
			Piece->Move(0,-1,0);
	}
	if(keyc[SDLK_UP] && !(keys[SDLK_UP])){
		keyc[SDLK_UP] = false;
	}

	if(keys[SDLK_DOWN]&& !keyc[SDLK_DOWN]){
		keyc[SDLK_DOWN] = true;
		Piece->Move(0,-1,0);
		if(TestPiece(Piece))
			Piece->Move(0,1,0);
	}
	if(keyc[SDLK_DOWN] && !(keys[SDLK_DOWN])){
		keyc[SDLK_DOWN] = false;
	}
	
	if(keys[SDLK_LEFT]&& !keyc[SDLK_LEFT]){
		keyc[SDLK_LEFT] = true;
		Piece->Move(-1,0,0);
		if(TestPiece(Piece))
			Piece->Move(1,0,0);
	}
	if(keyc[SDLK_LEFT] && !(keys[SDLK_LEFT])){
		keyc[SDLK_LEFT] = false;
	}
	
	if(keys[SDLK_RIGHT]&& !keyc[SDLK_RIGHT]){
		keyc[SDLK_RIGHT] = true;
		Piece->Move(1,0,0);
		if(TestPiece(Piece))
			Piece->Move(-1,0,0);
	}
	if(keyc[SDLK_RIGHT] && !(keys[SDLK_RIGHT])){
		keyc[SDLK_RIGHT] = false;
	}
	//X Axis Rotate
	if(keys[SDLK_q]&& !keyc[SDLK_q]){
		keyc[SDLK_q] = true;
		Piece->Rotate(RotX);
		if(TestPiece(Piece)){
			Piece->Rotate(RotMX);
			Piece->pz = PZ;
		}
	}
	if(keyc[SDLK_q] && !(keys[SDLK_q])){
		keyc[SDLK_q] = false;
	}
	if(keys[SDLK_a]&& !keyc[SDLK_a]){
		keyc[SDLK_a] = true;
		Piece->Rotate(RotMX);
		if(TestPiece(Piece)){
			Piece->Rotate(RotX);
			Piece->pz = PZ;
		}
	}
	if(keyc[SDLK_a] && !(keys[SDLK_a])){
		keyc[SDLK_a] = false;
	}
	//Y Axis Rotate
	if(keys[SDLK_w]&& !keyc[SDLK_w]){
		keyc[SDLK_w] = true;
		Piece->Rotate(RotY);
		if(TestPiece(Piece)){
			Piece->Rotate(RotMY);
			Piece->pz = PZ;
		}
	}
	if(keyc[SDLK_w] && !(keys[SDLK_w])){
		keyc[SDLK_w] = false;
	}
	if(keys[SDLK_s]&& !keyc[SDLK_s]){
		keyc[SDLK_s] = true;
		Piece->Rotate(RotMY);
		if(TestPiece(Piece)){
			Piece->Rotate(RotY);
			Piece->pz = PZ;
		}
	}
	if(keyc[SDLK_s] && !(keys[SDLK_s])){
		keyc[SDLK_s] = false;
	}
	//Z Axis Rotate
	if(keys[SDLK_e]&& !keyc[SDLK_e]){
		keyc[SDLK_e] = true;
		Piece->Rotate(RotZ);
		if(TestPiece(Piece)){
			Piece->Rotate(RotMZ);
			Piece->pz = PZ;
		}
	}
	if(keyc[SDLK_e] && !(keys[SDLK_e])){
		keyc[SDLK_e] = false;
	}
	if(keys[SDLK_d]&& !keyc[SDLK_d]){
		keyc[SDLK_d] = true;
		Piece->Rotate(RotMZ);
		if(TestPiece(Piece)){
			Piece->Rotate(RotZ);
			Piece->pz = PZ;
		}
	}
	if(keyc[SDLK_d] && !(keys[SDLK_d])){
		keyc[SDLK_d] = false;
	}
	if(keys[SDLK_SPACE]&& !keyc[SDLK_SPACE]){
		keyc[SDLK_SPACE] = true;
		fast = true;
	}
	if(keyc[SDLK_SPACE] && !(keys[SDLK_SPACE])){
		keyc[SDLK_SPACE] = false;
	}
	if(keys[SDLK_n]&& !keyc[SDLK_n]){
		keyc[SDLK_n] = true;
		NewPiece();
	}
	if(keyc[SDLK_n] && !(keys[SDLK_n])){
		keyc[SDLK_n] = false;
	}

}

void CGame::Tick(unsigned long t)//Do a frame
{
	heldticktime += t;
	
	if(!fast){
		if(heldticktime >= Speed){
			heldticktime = 0;
			Piece->Move(0,0,1);
			Test();
		}
	}
	else{//Else if fastmod
		if(heldticktime >= (50)){
			heldticktime = 0;
			Piece->Move(0,0,1);
			Test();
		}
	}
}

//Constructor
CGame::CGame(int h,int w,int d,int l,int s)
{
	GenerateLines(h,w,d);
	fast = false;
	Speed = SetSpeed(l);
	Level = l;
	LevelCount = 0;
	Score = 0;
	int size = h*(w*d);
	
	board = new int [size];
	//Assign Zero to array
	for(int c = 0;c<size;c++)
		board[c] = 0;
	//Pflog(&Log,"Size should be:%d Is:%d\n",size,board.size());
	this->Piece = new CPiece;
	Set = s;
	NewPiece();
	heldticktime = 0;
	Active = false;
	UpdateScoreMod();
}
void CGame::Restart(int h,int w,int d,int l,int s)
{
	GenerateLines(h,w,d);
	fast = false;
	Speed = SetSpeed(l);
	Level = l;
	LevelCount = 0;
	Score = 0;
	int size = h*(w*d);
	if(board != NULL)delete[] board;
	Set = s;
	board = new int [size];
	//Assign Zero to array
	for(int c = 0;c<size;c++)
		board[c] = 0;
	//Pflog(&Log,"Size should be:%d Is:%d\n",size,board.size());
	if(Piece != NULL)delete Piece;
	this->Piece = new CPiece;
	NewPiece();
	heldticktime = 0;
	Active = true;
	UpdateScoreMod();
}
void CGame::UpdateScoreMod()
{
	int t,diff;
	t = h*w*d;
	diff = (HEIGHTMAX*WIDTHMAX*DEPTHMAX)-t;
	ScoreMod = (int)((Level+1)*Set*diff)/15;

}
void CGame::NewPiece()
{
	int e = NUMBLOCKS;
	srand( (unsigned)time( NULL ) );
	int donePiece = 0;
	int PMAX,PMIN;
	switch(Set){
	case SET_FLAT:
		PMIN = 0;
		PMAX = 8;
		break;
	case SET_SIMPLE:
		PMIN = 0;
		PMAX = 20;
		break;
	case SET_COMPLEX:
		PMIN = 21;
		PMAX = 34;
		break;
	default:
		PMIN = 0;
		PMAX = 8;
		break;
	};
	while(!donePiece){
	e = (int)(rand()%NUMBLOCKS);
	while(e<PMIN || e >PMAX)	//Get block from proper set
		e = (int)(rand()%NUMBLOCKS);
	Piece->AssignBlock(blocks[e],blocksLength[e]);
	fast = false;
	Piece->FindMaxDimentions();
	if(!(	Piece->MaxDepth > d || //Probably wont happen but why not
			Piece->MaxDepth > h ||
			Piece->MaxDepth > w ||
			Piece->MaxHeight > d || 
			Piece->MaxHeight > h || 
			Piece->MaxHeight > w || 
			Piece->MaxWidth > d ||
			Piece->MaxWidth > h ||
			Piece->MaxWidth > w))
		donePiece = 1;
	}
	Piece->pz = -1+Piece->MaxDepth;
}
void CGame::Test()
{
	int x,y,z,ret = 1;
	if(Piece->pz <= 0+Piece->MaxDepth){
		if(TestPiece(Piece) == 1){
			//Game Over
			if(CheckHighScores(Score) >= 0){
				Gamestate = ENTER_HIGHSCORE;
				//Score += (int)(Piece->Psize+1)*ScoreMod;
				Score += (int)(Piece->Psize+1)*(Level+1)*Set;
				Active = 0;
			}
			else{
				Gamestate = HIGHSCORES;
				Active = 0;
			}
			return;
		}
	}
	else{
		if(TestPiece(Piece) == 1){
			for(x = Piece->LowX;x<Piece->MaxWidth;x++){
				for(y = Piece->LowY;y<Piece->MaxHeight;y++){
					for(z = Piece->LowZ;z<Piece->MaxDepth;z++){
						if(Piece->block[x][y][z])
							board[BoardPos(x+Piece->px,y+Piece->py,Piece->pz-z)] = 1;
					}
				}
			}
			//Score += (int)(Piece->Psize+1)*ScoreMod;
			Score += (int)(Piece->Psize+1)*(Level+1)*Set;
			LevelCount += (Piece->Psize+1)*Set;
			if(LevelCount > LEVELINCMAX){
				Level++;
				if(Level > LEVELMAX)
					Level = LEVELMAX;
				LevelCount = 0;
				UpdateScoreMod();
			}
			if(SoundOpt)
				BlockHit.Play();
			NewPiece();
		}
	}
	while(ret == 1){
		ret = TestBoard();
	}
}

int CGame::TestPiece(CPiece *P)
{
	int px,py,p;
	for(px = 0;px<MAXBLOCKS;px++){
		for(py = 0;py<MAXBLOCKS;py++){
			if(P->block[px][py][P->LowZ]){
				p = BoardPos(px+P->px,py+P->py,P->LowZ+P->pz+1);
				if(P->LowZ+P->pz+1 >= dp)
					return 1;
				if((board[p])){
					return 1;
				}
			}
		}
	}
	return 0;
}
int BoardPos(int x,int y,int z)
{
	return x+(y*wp)+(z*(hp*wp));
}
void CGame::DrawBoard()
{
	int x,y,z;
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LINE_SMOOTH);
//	glEnable(GL_LIGHTING);
	glTranslatef(0,0,DSIZE+xspace);
	LastZ = d;
	for( x = 0;x<wp;x++){
		for( y = 0;y<hp;y++){
			for( z = 0;z<dp;z++){
				if(board[BoardPos(x,y,z)]){
					SetColor(d-z);
					if(LastZ < z)//For game summary
						LastZ = z;
					DrawPicCube((GLfloat)x*zspace,(GLfloat)y*yspace,(GLfloat)(d-z-1)*xspace,zspace,yspace,xspace);
					//DrawPicCube(0,0,0,zspace,yspace,xspace);
				}
					
			}
		}
	}
	glLineWidth(2.0f);
	glColor3f(1,1,1);
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	for( x = 0;x<wp;x++){
		for( y = 0;y<hp;y++){
			for( z = 0;z<dp;z++){
				if(board[BoardPos(x,y,z)])
					DrawPicCube((GLfloat)x*zspace,(GLfloat)y*yspace,(GLfloat)(d-z-1)*xspace,zspace,yspace,xspace);
					//DrawPicCube(0,0,0,zspace,yspace,xspace);
					
			}
		}
	}
	glLineWidth(1.0f);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);//For blocks TODO: change to make an option
	glTranslatef(0,0,-DSIZE-xspace);
	glDisable(GL_LIGHTING);
	glColor3f(0,1,0);
}
void SetColor(int z)
{
	int c = z % 10;
	switch(c){
	case 1:
		glColor3f(0.0f,0.0f,1.0f);
		break;
	case 2:
		glColor3f(1.0f,0.0f,0.0f);
		break;
	case 3:
		glColor3f(0.0f,1.0f,0.0f);
		break;
	case 4:
		glColor3f(1.0f,1.0f,0.0f);
		break;
	case 5:
		glColor3f(1.0f,0.0f,1.0f);
		break;
	case 6:
		glColor3f(0.0f,1.0f,1.0f);
		break;
	case 7:
		glColor3f(1.0f,1.0f,0.5f);
		break;
	case 8:
		glColor3f(1.0f,0.5f,1.0f);
		break;
	case 9:
		glColor3f(0.5f,1.0f,1.0f);
		break;
	default:
		glColor3f(1.0f,0.0f,0.0f);
		break;
	};
}
//Set speed for different levels
int SetSpeed(int l)
{
	switch(l){
	case 1:
		return 1800;
		break;
	case 2:
		return 1600;
		break;
	case 3:
		return 1350;
		break;
	case 4:
		return 1200;
		break;
	case 5:
		return 1050;
		break;
	case 6:
		return 900;
		break;
	case 7:
		return 600;
		break;
	case 8:
		return 400;
		break;
	case 9:
		return 200;
		break;
	default:
		return 1000;
		break;
	};
}
int CGame::TestBoard()
{
	int z = 0;
	int x,y;
	int Failed = 0;
	int *t1 = NULL;//Board
	int test = 0;
	int size = h*(w*d);
	int xd,yd,zd,Rz = 0;
	while(1){
		Failed = 0;
		for(x = 0;x<w;x++){
			for(y = 0;y<h;y++){
				fflush(stdout);
				if(Failed == 0 && board[BoardPos(x,y,z)]){//if piece
					if((x == w-1) && (y == h-1)){//if finished
						for(xd = 0;xd<w;xd++){//clear row
							for(yd = 0;yd<h;yd++){
									board[BoardPos(xd,yd,z)] = 0;
							}
						}		
						t1 = new int[size];
						for(xd = 0;xd<size;xd++){
							t1[xd] = 0;
						}
						for(xd = 0;xd<size;xd++){
							if(board[xd] == 1)
							t1[xd] = 1;
						}
						for(xd = 0;xd<w;xd++){//Move back all else
							for(yd = 0;yd<h;yd++){
								for(zd = z;zd>0;zd--){
									t1[BoardPos(xd,yd,zd)] = board[BoardPos(xd,yd,zd-1)];
								}
							}
						}					
						for(xd = 0;xd<size;xd++){
							board[xd] = t1[xd];
							if(board[xd])
								test = 1;
						}				
						if(test == 0){
							if(SoundOpt)
								LastRow.Play();
							printf("Well Done\n");
						}
						else
							if(SoundOpt)
								RowDone.Play();
						if(t1 != NULL)delete[] t1;
						return 1;
					}
				}
				else{//else reset
					Failed = 1;

				}
			}
		}
		z++;
		if(z>=d)
		{
			if(t1 != NULL)delete[] t1;
			return 0;
		}
	}
	if(t1 != NULL)delete[] t1;
	return 0;
}
void CGame::DrawScore()
{
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,640,0,480,-100,100);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	MenuTex.BindTexture();
	glPrint(Height,0,1,"Score: %d",Score);

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
}
void LoadHighScores(char *file)
{
	FILE *f;
	f = fopen(file,"r");
	if(!f){
		printf("Error opening High Scores file\n");
		return;
	}

	char buff[64];
	int count = 0,count2 = 0;

		for(int Num = 0;Num < 10;Num++){
			fgets(buff,64,f);
			
			
			HighScores[Num] = atoi(buff);
			for(count = 0;count<64;count++)
				buff[count] = 0;
			
			fgets(buff,64,f);
			if(buff[strlen(buff)-1] == 10)
				buff[strlen(buff)-1] = 0;
			sprintf(HighScoreName[Num],"%s",buff);
		}
	if(fclose(f)!= 0)
		printf("Problem closing file\n");

}
void SaveHighScores(char *file)
{
	FILE *f;
	f = fopen(file,"w");
	if(!f){
		printf("Error opening High Scores file\n");
		return;
	}

	int count = 0,count2 = 0;

		for(int Num = 0;Num < 10;Num++){
			fprintf(f,"%d\n",HighScores[Num]);
			fprintf(f,"%s\n",HighScoreName[Num]);
		}
	if(fclose(f)!= 0)
		printf("Problem closing file\n");
}
void CGame::DrawHighScores()
{
	int count;
	//int count2;
	//int numDots;
	char buff[64];
	//char buff2[64];
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,640,0,480,-1,1);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
	glColor3f(0,0,0);
	DrawQuad2d(0,480,640,480,NULL);

	glColor3f(0,1,0);
/*	for(count = 0;count < 10;count++){
		sprintf(buff2,"%d",HighScores[count]);
		numDots = 100-(strlen(HighScoreName[count])-strlen(buff2));
		for(count2 = 0;count<numDots;count2++){
			buff2[count2] = '.';
		}
		sprintf(buff,"%s%s%d",HighScoreName[count],buff2,HighScores[count]);
		glPrint(10,Height-(32*count),1,buff);
	}*/
	if(keys[SDLK_ESCAPE]&& !keyc[SDLK_ESCAPE]){
		keyc[SDLK_ESCAPE] = true;
		Gamestate = GAME_MENU;
	}
	if(keyc[SDLK_ESCAPE] && !(keys[SDLK_ESCAPE])){
		keyc[SDLK_ESCAPE] = false;
	}
	glPrint(10,Height-25,0,"____    _____");	
	glPrint(10,Height-20,0,"Name    Score");	
	for(count = 0;count < 10;count++){
		sprintf(buff,"%s",HighScoreName[count]);//must do else get funny char at end
		
		glPrint(10,Height-45-(32*count+1),0,"%d)%s   %d",count+1,buff,HighScores[count]);
	}
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing	


}

int CheckHighScores(int n)
{

	if(n <=HighScores[9])
		return -1;
	for(int Num = 9;Num >= 0;Num--){
		if(n <= HighScores[Num]){
			return Num+1;
		}
	}
	return 0;
	
}
void MoveHighScores(int score,char *name,int Pos)
{
	printf("Well done %s, Your score of %d is number %d on the High Score list\n",
			name,score,Pos);
	for(int Num = 9;Num>=Pos;Num--)
	{
		HighScores[Num] = HighScores[Num-1];
		sprintf(HighScoreName[Num],"%s",HighScoreName[Num-1]);
	}
	HighScores[Pos] = score;
	sprintf(HighScoreName[Pos],"%s",name);
	SaveHighScores("data\\HiScores.dat");
}
void DrawEnterScore()
{

	static char buff[15];
	static int CurrentPos = 0;
	int count;
/*Check Keys*/
	for(count = SDLK_a;count < SDLK_z+1;count++){
		if(keys[count]&& !keyc[count]){
			keyc[count] = true;
			if(strlen(buff) >= 12)
				//do nothing
				;
			else{
				if(keys[SDLK_LSHIFT]||keys[SDLK_RSHIFT])
			   	   buff[CurrentPos] = count-'a'+'A';
			   else
					buff[CurrentPos] = count;
			   CurrentPos++;
			}

		}
		if(keyc[count] && !(keys[count])){
			keyc[count] = false;
		}
	}
	for(count = SDLK_0;count < SDLK_9+1;count++){
		if(keys[count]&& !keyc[count]){
			keyc[count] = true;
			if(strlen(buff) >= 12)
				//do nothing
				;
			else{
				buff[CurrentPos] = count;
			   CurrentPos++;
			}

		}
		if(keyc[count] && !(keys[count])){
			keyc[count] = false;
		}
	}
	if(keys[SDLK_RETURN]&& !keyc[SDLK_RETURN]){
		keyc[SDLK_RETURN] = true;
			MoveHighScores(Score,buff,CheckHighScores(Score));
			//Reset all
			memset(&buff,0,sizeof(buff));
			CurrentPos = 0;
			Gamestate = HIGHSCORES;
			return;
	}
	if(keyc[SDLK_RETURN] && !(keys[SDLK_RETURN])){
			keyc[SDLK_RETURN] = false;
	}
	if(keys[SDLK_BACKSPACE]&& !keyc[SDLK_BACKSPACE]){
			keyc[SDLK_BACKSPACE] = true;
			CurrentPos--;
			buff[CurrentPos] = NULL;

		}
		if(keyc[SDLK_BACKSPACE] && !(keys[SDLK_BACKSPACE])){
			keyc[SDLK_BACKSPACE] = false;
	}
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,640,0,480,-1,1);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
	glColor3f(0,0,0);
	DrawQuad2d(0,480,640,480,NULL);

	glColor3f(0,1,0);
	glPrint(200,290,1,"Please Enter Name");	
	glPrint(250,270,1,buff);	
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing	


}
void CheckHelpKeys();
#define NUMHELPPAGES 4
int Helppage = 1;
void DrawHelp()
{
	#define	X 30 
	#define CHARHEIGHT 16
	#define	Y(row) (390-(row*CHARHEIGHT))
	char buff[128];
	int row = 0;
	
	CheckHelpKeys();
	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,640,0,480,-100,100);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
	
	glColor3f(1 ,1,1);
	FontText.BindTexture();
	glEnable(GL_TEXTURE_2D);
	DrawQuad2d(0,480,640,480,&MenuTex);				//nice background
	switch(Helppage)
	{
		default:
		case 1:
			glSPrint(0.9f,640,480,X,Y(0),0,"Welcome to SDLBLOCK version 1.1");
			glSPrint(0.9f,640,480,X,Y(1),0,"-------------------------------");
			glSPrint(0.9f,640,480,X,Y(2),0,"Push Space to Quit to menu");
			glSPrint(0.9f,640,480,X,Y(4),0,"Push Right To go to Next page(if there is one)");
			glSPrint(0.9f,640,480,X,Y(6),0,"Push Left To go to Previous page");
			break;
		case 2:
			glSPrint(0.9f,640,480,X,Y(0),0,"Keys");
			glSPrint(0.9f,640,480,X,Y(1),0,"----");
			glSPrint(0.9f,640,480,X,Y(2),0,"Push Escape to Quit to menu");
			glSPrint(0.9f,640,480,X,Y(4),0,"Push Up/Down/Right/Left to move the piece");
			glSPrint(0.9f,640,480,X,Y(6),0,"Push Q and A to rotate along the X axis");
			glSPrint(0.9f,640,480,X,Y(8),0,"Push W and S to rotate along the Y axis");
			glSPrint(0.9f,640,480,X,Y(10),0,"Push Q and A to rotate along the Z axis");			
			break;
		case 3:
			glSPrint(0.9f,640,480,X,Y(0),0,"To change resolution and fullscreen");
			glSPrint(0.9f,640,480,X,Y(1),0,"-----------------------------------");
			glSPrint(0.9f,640,480,X,Y(2),0,"To run without fullscreen add a --nofullscreen");
			glSPrint(0.9f,640,480,X,Y(3),0,"flag to game command");
			glSPrint(0.9f,640,480,X,Y(4),0,"To change resolution...");
			glSPrint(0.9f,640,480,X,Y(6),0,"Open res.dat in the data directory");
			glSPrint(0.9f,640,480,X,Y(8),0,"Change the first line to the width");
			glSPrint(0.9f,640,480,X,Y(10),0,"Change the second line to the height");
			glSPrint(0.9f,640,480,X,Y(12),0,"Game Should work over 640x480 resolution");
			glSPrint(0.9f,640,480,X,Y(14),0,"Be careful not to remove any data files or ");
			glSPrint(0.9f,640,480,X,Y(15),0,"change them improperly as this may cause the");
			glSPrint(0.9f,640,480,X,Y(16),0,"game to crash");
			break;
		case 4:
			glSPrint(0.9f,640,480,X,Y(0),0,"Thanks...");
			glSPrint(0.9f,640,480,X,Y(1),0,"---------");			
			glSPrint(0.9f,640,480,X,Y(2),0,"Nehe and his great OpenGL tutorials");
			glSPrint(0.9f,640,480,X,Y(3),0,"Digiben at Gametutorials.com for his tutorials");
			glSPrint(0.9f,640,480,X,Y(4),0,"GameHippo (First decent advice for the game)");
			glSPrint(0.9f,640,480,X,Y(5),0,"SDL and SDL_Mixer for the libraries");
			break;
	};
	sprintf(buff,"Page %d/%d",Helppage,NUMHELPPAGES);
	glSPrint(0.9f,640,480,640-(CHARHEIGHT*strlen(buff)),32,0,buff);
		

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing	

}
void CheckHelpKeys()
{
	if(keyc[' '] && !(keys[' '])){
		keyc[' '] = false;
	}

	if(keys[' ']&& !keyc[' ']){
		keyc[' '] = true;
		Gamestate = GAME_MENU;
		Helppage = 1;
	}
	if(keyc[SDLK_RIGHT] && !(keys[SDLK_RIGHT])){
		keyc[SDLK_RIGHT] = false;
	}

	if(keys[SDLK_RIGHT]&& !keyc[SDLK_RIGHT]){
		keyc[SDLK_RIGHT] = true;
		if(Helppage < NUMHELPPAGES)
			Helppage++;
	}
	if(keyc[SDLK_LEFT] && !(keys[SDLK_LEFT])){
		keyc[SDLK_LEFT] = false;
	}

	if(keys[SDLK_LEFT]&& !keyc[SDLK_LEFT]){
		keyc[SDLK_LEFT] = true;
		if(Helppage > 1)
			Helppage--;
	}
}
