/*
   Copyright (C) 2002 Dylan Yudaken(d_yud@hotmail.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/

#ifndef _H_SDLBLOCK_DEF
#define _H_SDLBLOCK_DEF
#define MAXBLOCKS 5
#define NUMBLOCKS 35
#define LEVELINCMAX 250
#define LEVELMAX 9
#define WIDTHMIN 3
#define WIDTHMAX 7
#define HEIGHTMIN 3
#define HEIGHTMAX 7
#define DEPTHMIN 6 
#define DEPTHMAX 18
extern int blocks[NUMBLOCKS][MAXBLOCKS*MAXBLOCKS*MAXBLOCKS];
extern int blocksLength[NUMBLOCKS];
#define DSIZE 10.0f
#define HALFDSIZE 5.0f
#define USIZE 5.0f
#define HALFUSIZE 2.5f
enum {
	DUP,
	DLEFT,
	DRIGHT,
	DDOWN,
	DFORWARD,
	DBACKWARD
};
enum {
	SET_FLAT = 1,
	SET_SIMPLE = 2,
	SET_COMPLEX = 3
};
enum RotType{
	RotX,
	RotMX,
	RotY,
	RotMY,
	RotZ,
	RotMZ
};
enum{
	GAME_MAIN,
	GAME_MENU,
	HIGHSCORES,
	ENTER_HIGHSCORE,
	GAME_HELP
};
extern int Gamestate;
//Function Prototypes
void LoadHighScores(char *file);
int CheckHighScores(int n);
void MoveHighScores(int score,char *name,int Pos);
void DrawEnterScore();
void DrawHelp();

//Score Stuff
extern int Score;
extern int HighScores[10];
extern char HighScoreName[10][20];



class CPiece{
public:
	int rx,ry,rz; //Rotation
	int px,py,pz; //Position
	int MaxWidth,MaxHeight,MaxDepth,LowX,LowY,LowZ; //For Moving
	int block[MAXBLOCKS][MAXBLOCKS][MAXBLOCKS];
	void Rotate(RotType r);
	void Move(int x,int y,int z);
	int FindMaxDimentions();
	int DeleteMem();
	void AssignBlock(int *Directions,int size);
	void Reset();
	void Draw();
	int Psize;//Size of piece
	int *PDir;//Directions to Draw it
};

class CGame{
public:
	CGame(int h,int w,int d, int l,int s);//Con
	void Restart(int h,int w,int d, int l,int s);//Restart game

	//OLDvector<int> board;//Board
	int *board;
	void CheckGameKeys();//Check all game related keys
	int h,w,d;				//board size
	int Set;
	int ScoreMod;			//scoring modifier
	int SoundOpt;			//Do we have sound???
	int GhostBlock;			//Do we have a helping block???
	void UpdateScoreMod();//update scoring modifier
	int Level,LevelCount;				//last piece
	int LastZ;				//last pieces Z
	int GenerateLines(int h,int w,int d); //gen board display list
	void RenderLines();					  //ren board display list
	void Test();//Test for collision
	int TestPiece(CPiece *P);//Test piece 
	int TestBoard();
	void NewPiece();//Add new Piece
	void DrawBoard();
	void DrawHelpPiece(CPiece *P);
	CPiece *Piece;
	//Timer Stuff
	int fast;
	void Tick(unsigned long t);// 1 frame
	unsigned long Speed;
	unsigned long heldticktime;
	//Scoring Stuff
	void DrawScore();
	void DrawHighScores();
	//Playing or Not???
	int Active;
};

class GLBPos{
public:
	int x,y,z;
	GLBPos(int px,int py,int pz){
		x = px;
		y = py;
		z = pz;
	}
	GLBPos(const GLBPos &p){
		x = p.x;
		y = p.y;
		z = p.z;
	}
	GLBPos(){
		x = y = z = 0;
	}

};
GLBPos RotatePos(RotType r,GLBPos p);
void SetColor(int z);
int SetSpeed(int l);
/*-------------------------------------------------------*/
//				Textures and Sounds
/*-------------------------------------------------------*/
extern CIndivSound BlockHit;
extern CIndivSound LastRow;
extern CIndivSound RowDone;
extern CIndivTexture MenuTex;



























#endif
