/*
   Copyright (C) 2002 Dylan Yudaken(d_yud@hotmail.com)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

   See gpl.txt for more information regarding the GNU General Public License.
*/

#include "common.h"


/*-------------------------------------------------------
					General Globals
-------------------------------------------------------*/
unsigned long ticktime;
int Width = 640;
int Height = 480;
int sceneno = 2;
int	active=TRUE;		// Window Active Flag Set To TRUE By Default
int	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default
int keys[256];
int keyc[256];
CIndivTexture MenuTex;
CIndivSound BlockHit;
CIndivSound LastRow;
CIndivSound RowDone;
int Gamestate;
CLog Log;
CGame *Game;
CMenu Menu;
GLfloat ycount,xcount,zcount = 0.0f;
GLfloat xrot, yrot, zrot = 0.0f;
int blending = FALSE;
int    blend;						// Blending OFF/ON? ( NEW )
GLuint Tlist[100];
const float piover180 = 0.0174532925f;
int TexCount = 0;
CTexture CTexture;
CGl GL;
extern GLuint base;
void LoadResolution();
int DrawGLScene(GLvoid)						// Here's Where We Do All The Drawing
{
	static int count; //Frame Counter DELETE
	static GLfloat xp,yp,zp;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();// Reset The View


	//Do this anyway so we get the board always
	glTranslatef(xp,yp,-12.6f+zp);//Get to a nice pos
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);//For blocks TODO: change to make an option
	glColor3f(1,0,0);
	Game->RenderLines();
	Game->DrawBoard();
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glColor4f(0,1,0,0.8f);//bit trans
	Game->Piece->Draw();
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);//For blocks TODO: change to make an option
	Game->DrawHelpPiece(Game->Piece);
	glColor4f(1,1,1,1);
	Game->DrawScore();
	switch(Gamestate){
		case GAME_MAIN:
			Game->CheckGameKeys();
			Game->Tick(ticktime);
			if(keys[SDLK_ESCAPE]&& !keyc[SDLK_ESCAPE]){
				keyc[SDLK_ESCAPE] = true;
				Gamestate = GAME_MENU;
			}
			if(keyc[SDLK_ESCAPE] && !(keys[SDLK_ESCAPE])){
				keyc[SDLK_ESCAPE] = false;
			}
			break;
		case GAME_MENU:
			
			Menu.Draw();
			break;
		case HIGHSCORES:
			Game->DrawHighScores();
			break;
		case ENTER_HIGHSCORE:
			DrawEnterScore();
			break;
		case GAME_HELP:
			DrawHelp();
			break;
		default:
			//shouldnt happen
			break;
	};

	
	SDL_GL_SwapBuffers( );				// Swap Buffers (Double Buffering)
	count++;
	return 1;
}
void InitGame()
{
//	char buff[128];
	//MUST do this first in case other things want to access it
	if(Game == NULL) delete Game;
	Game = new CGame(3,3,11,9,1);	//Start game
	
	Pflog(&Log,"Loading Textures....\n");
	FontText.LoadGLTexture("Font.bmp");
	MenuTex.LoadGLTexture("MenuTex.bmp");
	BuildFont();
	Pflog(&Log,"Loading Sounds....\n");
	BlockHit.Load("BlockHit.wav");
	LastRow.Load("LastRow.wav");
	RowDone.Load("RowDone.wav");	
	Pflog(&Log,"Loading Menu....\n");
	Menu.init();
	Pflog(&Log,"Loading High Scores....\n");
	LoadHighScores("data\\HiScores.dat");
	Gamestate = GAME_MENU;
}
int main(int argc, char *argv[])
{
	Log.InitLog();
	int n = argc;
	int c;
	printf("Welcome to SDLBlock\n");
	printf("-------------------\n");
	printf("To not use fullscreen use --nofullscreen flag\n");
	printf("If you have errors please try this first\n");
	if(n > 1){
		for(c = 1;c<argc;c++)
			if(strcmp(argv[c],"--nofullscreen") == 0){
				printf("No fullscreen\n");
				fullscreen = false;
			}
	}
	LoadResolution();
	MainLoop();
	return 1;
}
void LoadResolution()
{
	char buffer[64];
	sprintf(buffer,"%s%s",DataDir,"res.dat");
	FILE *file = fopen(buffer,"r");
	
	fgets(buffer,64,file);//Option Width
	Width = atoi(buffer);
	fgets(buffer,64,file);//Option Height
	Height = atoi(buffer);

	fclose(file);
}


void RestartGame(int h,int w,int d,int l,int s)
{
	Game->Restart(h,w,d,l,s);
	Pflog(&Log,"Starting new Game H:%d W:%d D:%d L:%D\n",h,w,d,l);
}
	






